/* 
 * gs_filefunc.h
 *
 * Common file functions
 * 
 * by immi ( immi at gmx . org )
 * by eis_os (cim bytetransfer.de)
 *
 * last update: 27.05.2012
 */

#ifndef GS_FILEFUNC_H
#define	GS_FILEFUNC_H

#ifdef	__cplusplus
extern "C" {
#endif
    
#if defined(_WIN32)
#  define OPEN_FLAGS (_O_BINARY | O_RDONLY)
#  define CREAT_FLAGS (_O_BINARY | O_CREAT | O_WRONLY | O_TRUNC)
#  define PATH_DELIMITER '\\'
#else // linux
#  define OPEN_FLAGS (O_RDONLY)
#  define CREAT_FLAGS (O_CREAT | O_WRONLY | O_TRUNC)
#  define PATH_DELIMITER '/'
#endif


void print(unsigned int indent_level, const char* fmt, ...);
void die(const char *fmt, ...)
    __attribute__ ((noreturn, format(printf, 1,2) ));
void die(const char *fmt, ...);

int file_exists(const char *file);
int fileordir_present(const char *file);
char* make_path(const char *path, const char* file, const char *suffix);
char* filename_noext(const char *file);

int gs_openfile(const char *filename, uint32_t type, char* filetypestr, unsigned int *size);
int gs_read(int fd, void* dst, unsigned int len, unsigned int *remaining);
int gs_read_chunk(int fd, struct gs_chunk *ch, unsigned int *remaining);


void* write_chunk_start(int fd, uint32_t chunktype);
unsigned int write_chunk_end(void* s);

void* fwrite_chunk_start(FILE* f, uint32_t chunktype);
unsigned int fwrite_chunk_end(void* s);
// Text Parser

// read buffer
#define BUFSIZE 32 * 1024

struct parser {
    int fd;
    char buf[BUFSIZE];
    char *cur;
    char *line;
    char* errinfo;
    unsigned int nline;
    ssize_t bytes_left;
};

void die_parser(struct parser* parser, const char* p, const char *fmt, ...);
float read_float(const char *p, char **end, struct parser* parser);
unsigned int read_uint(char *p, char **end, struct parser* parser);
void parser_fill_buffer(struct parser* parser);
void parser_read_line(struct parser *parser);

struct parser* init_parser(const char* filename);
#ifdef	__cplusplus
}
#endif

#endif	/* GS_FILEFUNC_H */

